import { Pipe, ChangeDetectorRef, NgModule } from '@angular/core';
import { combineLatest } from 'rxjs';

/**
 * @deprecated This will be removed in the next major version.
 */
const RemoteDataTags = {
    NotAsked: 'NotAsked',
    InProgress: 'InProgress',
    Failure: 'Failure',
    Success: 'Success'
};
class NotAsked {
    constructor() {
        this.tag = 'NotAsked';
    }
    /**
     * @deprecated This will be removed in the next major version, use the `notAsked()` constructor function instead.
     * @see notAsked
     */
    static of() {
        return new NotAsked();
    }
}
class InProgress {
    constructor(val) {
        this.val = val;
        this.tag = 'InProgress';
    }
    /**
     * @deprecated This will be removed in the next major version, use the `inProgress()` constructor function instead.
     * @see inProgress
     */
    static of(value) {
        return new InProgress(value);
    }
    value() {
        return this.val;
    }
}
class Failure {
    constructor(err, val) {
        this.err = err;
        this.val = val;
        this.tag = 'Failure';
    }
    /**
     * @deprecated This will be removed in the next major version, use the `failure()` constructor function instead.
     * @see failure
     */
    static of(err, val) {
        return new Failure(err, val);
    }
    value() {
        return this.val;
    }
    error() {
        return this.err;
    }
}
class Success {
    constructor(val) {
        this.val = val;
        this.tag = 'Success';
    }
    /**
     * @deprecated This will be removed in the next major version, use the `success()` constructor function instead.
     * @see success
     */
    static of(value) {
        return new Success(value);
    }
    value() {
        return this.val;
    }
}
// ----------------------------
//
//  Constructor functions
//
// ----------------------------
const notAsked = () => {
    return NotAsked.of();
};
const inProgress = (value) => {
    return InProgress.of(value);
};
const failure = (err, val) => {
    return Failure.of(err, val);
};
const success = (value) => {
    return Success.of(value);
};
// ----------------------------
//
//  Guards
//
// ----------------------------
const isNotAsked = (value) => {
    return (value !== null &&
        value !== undefined &&
        value.tag === 'NotAsked');
};
const isInProgress = (value) => {
    return (value !== null &&
        value !== undefined &&
        value.tag === 'InProgress');
};
const isFailure = (value) => {
    return (value !== null &&
        value !== undefined &&
        value.tag === 'Failure');
};
const isSuccess = (value) => {
    return (value !== null &&
        value !== undefined &&
        value.tag === 'Success');
};
const isRemoteData = (value) => {
    return ((value !== null &&
        value !== undefined &&
        value.tag === 'NotAsked') ||
        value.tag === 'InProgress' ||
        value.tag === 'Success' ||
        value.tag === 'Failure');
};
// ----------------------------
//
//  Folds
//
// ----------------------------
const fold = (onNotAsked, onInProgress, onFailure, onSuccess, rd) => {
    switch (rd.tag) {
        case 'NotAsked':
            return onNotAsked();
        case 'InProgress':
            return onInProgress(rd.value());
        case 'Failure':
            return onFailure(rd.error(), rd.value());
        case 'Success':
            return onSuccess(rd.value());
    }
};
const getOrElse = (rd, defaultValue) => {
    switch (rd.tag) {
        case 'Success':
            return rd.value();
        default:
            return defaultValue;
    }
};
// ----------------------------
//
//  Transformations
//
// ----------------------------
const map = (fn, rd) => isSuccess(rd) ? success(fn(rd.value())) : rd;
const mapFailure = (fn, rd) => isFailure(rd) ? failure(fn(rd.error())) : rd;

const assertIsRemoteData = (rd) => {
    if (!isRemoteData(rd)) {
        throw new Error(`Value "${rd}" is not a RemoteData<T> instance. Did you forget to use the async pipe? i.e: state$ | async | isInProgress`);
    }
};
const ɵ0 = assertIsRemoteData;
class IsNotAskedPipe {
    transform(rd) {
        if (rd === null || rd === undefined) {
            return false;
        }
        assertIsRemoteData(rd);
        return isNotAsked(rd);
    }
}
IsNotAskedPipe.decorators = [
    { type: Pipe, args: [{ name: 'isNotAsked' },] }
];
class IsInProgressPipe {
    transform(rd) {
        if (rd === null || rd === undefined) {
            return false;
        }
        assertIsRemoteData(rd);
        return isInProgress(rd);
    }
}
IsInProgressPipe.decorators = [
    { type: Pipe, args: [{ name: 'isInProgress' },] }
];
class AnyIsInProgressPipe {
    constructor(cd) {
        this.cd = cd;
        this._latestValue = false;
        this._subscription = null;
        this._rds$ = [];
    }
    transform(rds$) {
        if (rds$ === null || rds$ === undefined) {
            return false;
        }
        if (this._rds$ === rds$) {
            return this._latestValue;
        }
        else if (this._subscription) {
            this.dispose();
        }
        this._rds$ = rds$;
        this._subscription = combineLatest(rds$).subscribe(rds => {
            this._latestValue = rds.some(isInProgress);
            this.cd.markForCheck();
        });
        return this._latestValue;
    }
    ngOnDestroy() {
        this.dispose();
    }
    dispose() {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
        this._rds$ = [];
        this._latestValue = false;
    }
}
AnyIsInProgressPipe.decorators = [
    { type: Pipe, args: [{ name: 'anyIsInProgress', pure: false },] }
];
AnyIsInProgressPipe.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
class AnyIsNotAskedPipe {
    constructor(cd) {
        this.cd = cd;
        this._latestValue = false;
        this._subscription = null;
        this._rds$ = [];
    }
    transform(rds$) {
        if (rds$ === null || rds$ === undefined) {
            return false;
        }
        if (this._rds$ === rds$) {
            return this._latestValue;
        }
        else if (this._subscription) {
            this.dispose();
        }
        this._rds$ = rds$;
        this._subscription = combineLatest(rds$).subscribe(rds => {
            this._latestValue = rds.some(isNotAsked);
            this.cd.markForCheck();
        });
        return this._latestValue;
    }
    ngOnDestroy() {
        this.dispose();
    }
    dispose() {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
        this._rds$ = [];
        this._latestValue = false;
    }
}
AnyIsNotAskedPipe.decorators = [
    { type: Pipe, args: [{ name: 'anyIsNotAsked', pure: false },] }
];
AnyIsNotAskedPipe.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
class IsFailurePipe {
    transform(rd) {
        if (rd === null || rd === undefined) {
            return false;
        }
        assertIsRemoteData(rd);
        return isFailure(rd);
    }
}
IsFailurePipe.decorators = [
    { type: Pipe, args: [{ name: 'isFailure' },] }
];
class IsSuccessPipe {
    transform(rd) {
        if (rd === null || rd === undefined) {
            return false;
        }
        assertIsRemoteData(rd);
        return isSuccess(rd);
    }
}
IsSuccessPipe.decorators = [
    { type: Pipe, args: [{ name: 'isSuccess' },] }
];
class HasValuePipe {
    transform(rd) {
        if (isSuccess(rd)) {
            return true;
        }
        else if ((isInProgress(rd) || isFailure(rd)) &&
            rd.value() !== null &&
            rd.value() !== undefined) {
            return true;
        }
        else if (rd === undefined || rd === null) {
            return false;
        }
        assertIsRemoteData(rd);
        return false;
    }
}
HasValuePipe.decorators = [
    { type: Pipe, args: [{ name: 'hasValue' },] }
];
class GetSuccessPipe {
    transform(rd, defaultValue) {
        if (isSuccess(rd)) {
            return rd.value();
        }
        else if (rd === undefined || rd === null) {
            return defaultValue;
        }
        assertIsRemoteData(rd);
        return defaultValue;
    }
}
GetSuccessPipe.decorators = [
    { type: Pipe, args: [{ name: 'successValue' },] }
];
class GetInProgressPipe {
    transform(rd, defaultValue) {
        if (isInProgress(rd)) {
            return rd.value();
        }
        else if (rd === undefined || rd === null) {
            return defaultValue;
        }
        assertIsRemoteData(rd);
        return defaultValue;
    }
}
GetInProgressPipe.decorators = [
    { type: Pipe, args: [{ name: 'inProgressValue' },] }
];
class GetRemoteDataValuePipe {
    transform(rd) {
        if (rd === undefined || rd === null) {
            return undefined;
        }
        assertIsRemoteData(rd);
        return isInProgress(rd) || isSuccess(rd) || isFailure(rd)
            ? rd.value()
            : undefined;
    }
}
GetRemoteDataValuePipe.decorators = [
    { type: Pipe, args: [{ name: 'remoteDataValue' },] }
];
class GetFailureErrorPipe {
    transform(rd) {
        if (rd === undefined || rd === null) {
            return undefined;
        }
        assertIsRemoteData(rd);
        return isFailure(rd) ? rd.error() : undefined;
    }
}
GetFailureErrorPipe.decorators = [
    { type: Pipe, args: [{ name: 'failureError' },] }
];
class GetFailureValuePipe {
    transform(rd) {
        if (rd === undefined || rd === null) {
            return undefined;
        }
        assertIsRemoteData(rd);
        return isFailure(rd) ? rd.value() : undefined;
    }
}
GetFailureValuePipe.decorators = [
    { type: Pipe, args: [{ name: 'failureValue' },] }
];

const declarations = [
    IsNotAskedPipe,
    IsInProgressPipe,
    IsFailurePipe,
    IsSuccessPipe,
    HasValuePipe,
    GetSuccessPipe,
    GetFailureErrorPipe,
    GetFailureValuePipe,
    GetInProgressPipe,
    GetRemoteDataValuePipe,
    AnyIsInProgressPipe,
    AnyIsNotAskedPipe
];
class RemoteDataModule {
}
RemoteDataModule.decorators = [
    { type: NgModule, args: [{
                declarations,
                imports: [],
                exports: declarations
            },] }
];

/*
 * Public API Surface of lib
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AnyIsInProgressPipe, AnyIsNotAskedPipe, Failure, GetFailureErrorPipe, GetFailureValuePipe, GetInProgressPipe, GetRemoteDataValuePipe, GetSuccessPipe, HasValuePipe, InProgress, IsFailurePipe, IsInProgressPipe, IsNotAskedPipe, IsSuccessPipe, NotAsked, RemoteDataModule, RemoteDataTags, Success, failure, fold, getOrElse, inProgress, isFailure, isInProgress, isNotAsked, isRemoteData, isSuccess, map, mapFailure, notAsked, success, ɵ0 };
//# sourceMappingURL=ngx-remotedata.js.map
